@@ -1,7 +1,7 @@
 /*
  * The parse tree transformation module for SIP.
  *
- * Copyright (c) 2011 Riverbank Computing Limited <info@riverbankcomputing.com>
+ * Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
  *
  * This file is part of SIP.
  *
@@ -64,8 +64,7 @@ static void resolveFuncTypes(sipSpec *pt, moduleDef *mod, classDef *c_scope,
 static void resolvePySigTypes(sipSpec *,moduleDef *,classDef *,overDef *,signatureDef *,int);
 static void resolveVariableType(sipSpec *,varDef *);
 static void fatalNoDefinedType(scopedNameDef *);
-static void getBaseType(sipSpec *,moduleDef *,classDef *,argDef *);
-static void resolveType(sipSpec *,moduleDef *,classDef *,argDef *, int);
+static void resolveType(sipSpec *,moduleDef *,classDef *,argDef *,int);
 static void searchClassScope(sipSpec *,classDef *,scopedNameDef *,argDef *);
 static void searchMappedTypes(sipSpec *,moduleDef *,scopedNameDef *,argDef *);
 static void searchEnums(sipSpec *,scopedNameDef *,argDef *);
@@ -796,7 +795,8 @@ static void moveGlobalSlot(sipSpec *pt, moduleDef *mod, memberDef *gmd)
 
         if (mdhead == NULL)
         {
-            fatal("One of the arguments of ");
+            fatal("%s:%d: One of the arguments of ", od->sloc.name,
+                    od->sloc.linenr);
             prOverloadName(stderr, od);
             fatal(" must be a class or enum\n");
         }
@@ -811,14 +811,16 @@ static void moveGlobalSlot(sipSpec *pt, moduleDef *mod, memberDef *gmd)
         {
             if (second)
             {
-                fatal("The first argument of ");
+                fatal("%s:%d: The first argument of ", od->sloc.name,
+                        od->sloc.linenr);
                 prOverloadName(stderr, od);
                 fatal(" must be a class or enum\n");
             }
 
             if (mod != gmd->module && arg0->atype == enum_type)
             {
-                fatal("The first argument of ");
+                fatal("%s:%d: The first argument of ", od->sloc.name,
+                        od->sloc.linenr);
                 prOverloadName(stderr, od);
                 fatal(" must be a class\n");
             }
@@ -1366,7 +1368,7 @@ static void transformTypedefs(sipSpec *pt, moduleDef *mod)
     for (td = pt->typedefs; td != NULL; td = td->next)
         if (td->module == mod)
             if (td->ecd == NULL || !isTemplateClass(td->ecd))
-                getBaseType(pt, td->module, td->ecd, &td->type);
+                resolveType(pt, td->module, td->ecd, &td->type, FALSE);
 }
 
 
@@ -1440,7 +1442,7 @@ static void transformCasts(sipSpec *pt, classDef *cd)
     {
         classDef *dcd;
 
-        getBaseType(pt, cd->iff->module, cd, &al->arg);
+        resolveType(pt, cd->iff->module, cd, &al->arg, FALSE);
 
         if (al->arg.atype == class_type)
             dcd = al->arg.u.cd;
@@ -1582,6 +1584,8 @@ static void transformScopeOverloads(sipSpec *pt, classDef *c_scope,
                 {
                     ifaceFileDef *iff;
 
+                    fatal("%s:%d: ", od->sloc.name, od->sloc.linenr);
+
                     if (mt_scope != NULL)
                         iff = mt_scope->iff;
                     else if (c_scope != NULL)
@@ -1870,14 +1874,15 @@ static void resolveCtorTypes(sipSpec *pt,classDef *scope,ctorDef *ct)
     /* Handle any C++ signature. */
     if (ct->cppsig != NULL && ct->cppsig != &ct->pysig)
         for (a = 0; a < ct -> cppsig -> nrArgs; ++a)
-            getBaseType(pt, scope->iff->module, scope, &ct->cppsig->args[a]);
+            resolveType(pt, scope->iff->module, scope, &ct->cppsig->args[a],
+                    TRUE);
  
     /* Handle the Python signature. */
     for (a = 0; a < ct -> pysig.nrArgs; ++a)
     {
         argDef *ad = &ct -> pysig.args[a];
 
-        getBaseType(pt, scope->iff->module, scope, ad);
+        resolveType(pt, scope->iff->module, scope, ad, FALSE);
 
         if (!supportedType(scope,NULL,ad,FALSE) && (ct -> cppsig == &ct -> pysig || ct -> methodcode == NULL))
         {
@@ -1904,10 +1909,10 @@ static void resolveFuncTypes(sipSpec *pt, moduleDef *mod, classDef *c_scope,
     {
         int a;
 
-        getBaseType(pt, mod, c_scope, &od->cppsig->result);
+        resolveType(pt, mod, c_scope, &od->cppsig->result, TRUE);
 
         for (a = 0; a < od->cppsig->nrArgs; ++a)
-            getBaseType(pt, mod, c_scope, &od->cppsig->args[a]);
+            resolveType(pt, mod, c_scope, &od->cppsig->args[a], TRUE);
     }
  
     /* Handle the Python signature. */
@@ -1919,26 +1924,29 @@ static void resolveFuncTypes(sipSpec *pt, moduleDef *mod, classDef *c_scope,
     if (isSSizeReturnSlot(od->common))
         if ((res->atype != ssize_type && res->atype != int_type) || res->nrderefs != 0 ||
             isReference(res) || isConstArg(res))
-            fatal("%s slots must return SIP_SSIZE_T\n",
-                    od->common->pyname->text);
+            fatal("%s:%d: %s slots must return SIP_SSIZE_T\n", od->sloc.name,
+                    od->sloc.linenr, od->common->pyname->text);
 
     /* These slots must return int. */
     if (isIntReturnSlot(od->common))
         if (res->atype != int_type || res->nrderefs != 0 ||
             isReference(res) || isConstArg(res))
-            fatal("%s slots must return int\n", od->common->pyname->text);
+            fatal("%s:%d: %s slots must return int\n", od->sloc.name,
+                    od->sloc.linenr, od->common->pyname->text);
 
     /* These slots must return void. */
     if (isVoidReturnSlot(od->common))
         if (res->atype != void_type || res->nrderefs != 0 ||
             isReference(res) || isConstArg(res))
-            fatal("%s slots must return void\n", od->common->pyname->text);
+            fatal("%s:%d: %s slots must return void\n", od->sloc.name,
+                    od->sloc.linenr, od->common->pyname->text);
 
     /* These slots must return long. */
     if (isLongReturnSlot(od->common))
         if (res->atype != long_type || res->nrderefs != 0 ||
             isReference(res) || isConstArg(res))
-            fatal("%s slots must return long\n", od->common->pyname->text);
+            fatal("%s:%d: %s slots must return long\n", od->sloc.name,
+                    od->sloc.linenr, od->common->pyname->text);
 }
 
 
@@ -1955,27 +1963,31 @@ static void resolvePySigTypes(sipSpec *pt, moduleDef *mod, classDef *scope,
     {
         if (issignal)
         {
+            fatal("%s:%d: ", od->sloc.name, od->sloc.linenr);
+
             if (scope != NULL)
             {
                 fatalScopedName(classFQCName(scope));
                 fatal("::");
             }
 
-            fatal("%s() signals must return void\n",od -> cppname);
+            fatal("%s() signals must return void\n", od->cppname);
         }
 
-        getBaseType(pt, mod, scope, res);
+        resolveType(pt, mod, scope, res, FALSE);
 
         /* Results must be simple. */
         if (!supportedType(scope,od,res,FALSE) && (od -> cppsig == &od -> pysig || od -> methodcode == NULL))
         {
+            fatal("%s:%d: ", od->sloc.name, od->sloc.linenr);
+
             if (scope != NULL)
             {
                 fatalScopedName(classFQCName(scope));
                 fatal("::");
             }
 
-            fatal("%s() unsupported function return type - provide %%MethodCode and a %s signature\n",od -> cppname,(pt -> genc ? "C" : "C++"));
+            fatal("%s() unsupported function return type - provide %%MethodCode and a %s signature\n", od->cppname, (pt->genc ? "C" : "C++"));
         }
     }
 
@@ -1983,7 +1995,7 @@ static void resolvePySigTypes(sipSpec *pt, moduleDef *mod, classDef *scope,
     {
         argDef *ad = &pysig -> args[a];
 
-        getBaseType(pt, mod, scope, ad);
+        resolveType(pt, mod, scope, ad, FALSE);
 
         if (ad -> atype == slotcon_type)
             resolvePySigTypes(pt, mod, scope, od, ad->u.sa, TRUE);
@@ -1996,6 +2008,8 @@ static void resolvePySigTypes(sipSpec *pt, moduleDef *mod, classDef *scope,
         {
             if (!supportedType(scope,od,ad,FALSE))
             {
+                fatal("%s:%d: ", od->sloc.name, od->sloc.linenr);
+
                 if (scope != NULL)
                 {
                     fatalScopedName(classFQCName(scope));
@@ -2007,6 +2021,8 @@ static void resolvePySigTypes(sipSpec *pt, moduleDef *mod, classDef *scope,
         }
         else if (!supportedType(scope,od,ad,TRUE) && (od -> cppsig == &od -> pysig || od -> methodcode == NULL || (isVirtual(od) && od -> virthandler -> virtcode == NULL)))
         {
+            fatal("%s:%d: ", od->sloc.name, od->sloc.linenr);
+
             if (scope != NULL)
             {
                 fatalScopedName(classFQCName(scope));
@@ -2014,9 +2030,9 @@ static void resolvePySigTypes(sipSpec *pt, moduleDef *mod, classDef *scope,
             }
 
             if (isVirtual(od))
-                fatal("%s() unsupported function argument type - provide %%MethodCode, a valid %%VirtualCatcherCode and a valid C++ signature\n",od -> cppname);
+                fatal("%s() unsupported function argument type - provide %%MethodCode, a valid %%VirtualCatcherCode and a valid C++ signature\n", od->cppname);
 
-            fatal("%s() unsupported function argument type - provide %%MethodCode and a valid %s signature\n",od -> cppname,(pt -> genc ? "C" : "C++"));
+            fatal("%s() unsupported function argument type - provide %%MethodCode and a valid %s signature\n", od->cppname, (pt->genc ? "C" : "C++"));
         }
 
         if (scope != NULL)
@@ -2033,7 +2049,7 @@ static void resolveVariableType(sipSpec *pt, varDef *vd)
     int bad = TRUE;
     argDef *vtype = &vd->type;
 
-    getBaseType(pt, vd->module, vd->ecd, vtype);
+    resolveType(pt, vd->module, vd->ecd, vtype, FALSE);
 
     switch (vtype->atype)
     {
@@ -2848,16 +2864,6 @@ static void scopeDefaultValue(sipSpec *pt,classDef *cd,argDef *ad)
 }
 
 
-/*
- * Make sure a type is a base type.
- */
-static void getBaseType(sipSpec *pt, moduleDef *mod, classDef *c_scope,
-        argDef *type)
-{
-    resolveType(pt, mod, c_scope, type, FALSE);
-}
-
-
 /*
  * Resolve a type if possible.
  */
@@ -2904,7 +2910,7 @@ static void resolveType(sipSpec *pt, moduleDef *mod, classDef *c_scope,
         int sa;
 
         for (sa = 0; sa < type->u.sa->nrArgs; ++sa)
-            getBaseType(pt, mod, c_scope, &type->u.sa->args[sa]);
+            resolveType(pt, mod, c_scope, &type->u.sa->args[sa], FALSE);
     }
 
     /* See if the type refers to an instantiated template. */
@@ -2989,6 +2995,7 @@ static mappedTypeDef *instantiateMappedTypeTemplate(sipSpec *pt, moduleDef *mod,
             mappedtype_iface, NULL, type);
     mtd->iff->module = mod;
 
+    mtd->mtflags = mtt->mt->mtflags;
     mtd->doctype = templateString(mtt->mt->doctype, type_names, type_values);
 
     appendCodeBlockList(&mtd->iff->hdrcode,
