@@ -1,7 +1,7 @@
 /*
  * SIP library code.
  *
- * Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+ * Copyright (c) 2012 Riverbank Computing Limited <info@riverbankcomputing.com>
  *
  * This file is part of SIP.
  *
@@ -46,7 +46,7 @@ struct vp_values {
 static int check_size(PyObject *self);
 static int check_rw(PyObject *self);
 static int check_index(PyObject *self, SIP_SSIZE_T idx);
-#if PY_VERSION_HEX < 0x02060300
+#if PY_VERSION_HEX < 0x02060000
 static SIP_SSIZE_T get_value_data(PyObject *value, void **value_ptr);
 #endif
 #if PY_VERSION_HEX < 0x02050000
@@ -66,8 +66,6 @@ static int vp_convertor(PyObject *arg, struct vp_values *vp);
  */
 static PyObject *sipVoidPtr_ascapsule(sipVoidPtrObject *v, PyObject *arg)
 {
-    (void)arg;
-
     return PyCapsule_New(v->voidptr, NULL, NULL);
 }
 #endif
@@ -79,8 +77,6 @@ static PyObject *sipVoidPtr_ascapsule(sipVoidPtrObject *v, PyObject *arg)
  */
 static PyObject *sipVoidPtr_ascobject(sipVoidPtrObject *v, PyObject *arg)
 {
-    (void)arg;
-
     return PyCObject_FromVoidPtr(v->voidptr, NULL);
 }
 #endif
@@ -125,8 +121,6 @@ static PyObject *sipVoidPtr_asstring(sipVoidPtrObject *v, PyObject *args,
  */
 static PyObject *sipVoidPtr_getsize(sipVoidPtrObject *v, PyObject *arg)
 {
-    (void)arg;
-
 #if PY_MAJOR_VERSION >= 3
     return PyLong_FromSsize_t(v->size);
 #elif PY_VERSION_HEX >= 0x02050000
@@ -167,8 +161,6 @@ static PyObject *sipVoidPtr_setsize(sipVoidPtrObject *v, PyObject *arg)
  */
 static PyObject *sipVoidPtr_getwriteable(sipVoidPtrObject *v, PyObject *arg)
 {
-    (void)arg;
-
     return PyBool_FromLong(v->rw);
 }
 
@@ -205,19 +197,10 @@ static PyMethodDef sipVoidPtr_Methods[] = {
     {"setsize", (PyCFunction)sipVoidPtr_setsize, METH_O, NULL},
     {"getwriteable", (PyCFunction)sipVoidPtr_getwriteable, METH_NOARGS, NULL},
     {"setwriteable", (PyCFunction)sipVoidPtr_setwriteable, METH_O, NULL},
-    {NULL, NULL, 0, NULL}
+    {NULL}
 };
 
 
-/*
- * Implement bool() for the type.
- */
-static int sipVoidPtr_bool(PyObject *self)
-{
-    return (((sipVoidPtrObject *)self)->voidptr != NULL);
-}
-
-
 /*
  * Implement int() for the type.
  */
@@ -257,7 +240,7 @@ static PyNumberMethods sipVoidPtr_NumberMethods = {
     0,                      /* nb_negative */
     0,                      /* nb_positive */
     0,                      /* nb_absolute */
-    sipVoidPtr_bool,        /* nb_bool (Python v3), nb_nonzero (Python v2) */
+    0,                      /* nb_bool (Python v3), nb_nonzero (Python v2) */
     0,                      /* nb_invert */
     0,                      /* nb_lshift */
     0,                      /* nb_rshift */
@@ -408,18 +391,11 @@ static PySequenceMethods sipVoidPtr_SequenceMethods = {
     0,                      /* sq_concat */
     0,                      /* sq_repeat */
     sipVoidPtr_item,        /* sq_item */
-#if PY_VERSION_HEX >= 0x02050000
-    0,                      /* sq_slice */
-    0,                      /* sq_ass_item */
-    0,                      /* sq_ass_slice */
-#else
+#if PY_VERSION_HEX < 0x02050000
     sipVoidPtr_slice,       /* sq_slice */
     sipVoidPtr_ass_item,    /* sq_ass_item */
     sipVoidPtr_ass_slice,   /* sq_ass_slice */
 #endif
-    0,                      /* sq_contains */
-    0,                      /* sq_inplace_concat */
-    0,                      /* sq_inplace_repeat */
 };
 
 
@@ -479,7 +455,7 @@ static int sipVoidPtr_ass_subscript(PyObject *self, PyObject *key,
 {
     sipVoidPtrObject *v;
     Py_ssize_t start, size;
-#if PY_VERSION_HEX >= 0x02060300
+#if PY_VERSION_HEX >= 0x02060000
     Py_buffer value_view;
 #else
     Py_ssize_t value_size;
@@ -526,7 +502,7 @@ static int sipVoidPtr_ass_subscript(PyObject *self, PyObject *key,
         return -1;
     }
 
-#if PY_VERSION_HEX >= 0x02060300
+#if PY_VERSION_HEX >= 0x02060000
     if (PyObject_GetBuffer(value, &value_view, PyBUF_CONTIG_RO) < 0)
         return -1;
 
@@ -572,9 +548,9 @@ static PyMappingMethods sipVoidPtr_MappingMethods = {
 #endif
 
 
-#if PY_VERSION_HEX >= 0x02060300
+#if PY_VERSION_HEX >= 0x02060000
 /*
- * The buffer implementation for Python v2.6.3 and later.
+ * The buffer implementation for Python v2.6 and later.
  */
 static int sipVoidPtr_getbuffer(PyObject *self, Py_buffer *buf, int flags)
 {
@@ -663,7 +639,7 @@ static PyBufferProcs sipVoidPtr_BufferProcs = {
     sipVoidPtr_getsegcount, /* bf_getsegcount */
 #if PY_VERSION_HEX >= 0x02050000
     (charbufferproc)sipVoidPtr_getreadbuffer,   /* bf_getcharbuffer */
-#if PY_VERSION_HEX >= 0x02060300
+#if PY_VERSION_HEX >= 0x02060000
     sipVoidPtr_getbuffer,   /* bf_getbuffer */
     0                       /* bf_releasebuffer */
 #endif
@@ -765,20 +741,6 @@ PyTypeObject sipVoidPtr_Type = {
     0,                      /* tp_init */
     0,                      /* tp_alloc */
     sipVoidPtr_new,         /* tp_new */
-    0,                      /* tp_free */
-    0,                      /* tp_is_gc */
-    0,                      /* tp_bases */
-    0,                      /* tp_mro */
-    0,                      /* tp_cache */
-    0,                      /* tp_subclasses */
-    0,                      /* tp_weaklist */
-    0,                      /* tp_del */
-#if PY_VERSION_HEX >= 0x02060000
-    0,                      /* tp_version_tag */
-#endif
-#if PY_VERSION_HEX >= 0x03040000
-    0,                      /* tp_finalize */
-#endif
 };
 
 
@@ -787,16 +749,27 @@ PyTypeObject sipVoidPtr_Type = {
  */
 void *sip_api_convert_to_void_ptr(PyObject *obj)
 {
-    struct vp_values vp;
-
     if (obj == NULL)
     {
         PyErr_SetString(PyExc_TypeError, "sip.voidptr is NULL");
         return NULL;
     }
 
-    if (vp_convertor(obj, &vp))
-        return vp.voidptr;
+    if (obj == Py_None)
+        return NULL;
+
+    if (PyObject_TypeCheck(obj, &sipVoidPtr_Type))
+        return ((sipVoidPtrObject *)obj)->voidptr;
+
+#if defined(SIP_USE_PYCAPSULE)
+    if (PyCapsule_CheckExact(obj))
+        return PyCapsule_GetPointer(obj, NULL);
+#endif
+
+#if defined(SIP_SUPPORT_PYCOBJECT)
+    if (PyCObject_Check(obj))
+        return PyCObject_AsVoidPtr(obj);
+#endif
 
     return PyLong_AsVoidPtr(obj);
 }
@@ -884,7 +857,7 @@ static int check_index(PyObject *self, SIP_SSIZE_T idx)
 }
 
 
-#if PY_VERSION_HEX < 0x02060300
+#if PY_VERSION_HEX < 0x02060000
 /*
  * Get the address and size of the data from a value that supports the buffer
  * interface.
@@ -1013,38 +986,16 @@ static int vp_convertor(PyObject *arg, struct vp_values *vp)
         size = ((sipVoidPtrObject *)arg)->size;
         rw = ((sipVoidPtrObject *)arg)->rw;
     }
-#if PY_VERSION_HEX >= 0x02060300
-    else if (PyObject_CheckBuffer(arg))
-    {
-        Py_buffer view;
-
-        if (PyObject_GetBuffer(arg, &view, PyBUF_SIMPLE) < 0)
-            return 0;
-
-        ptr = view.buf;
-        size = view.len;
-        rw = !view.readonly;
-
-        PyBuffer_Release(&view);
-    }
-#endif
-#if PY_VERSION_HEX < 0x03000000
-    else if (PyObject_AsReadBuffer(arg, (const void **)&ptr, &size) >= 0)
-    {
-        rw = (Py_TYPE(arg)->tp_as_buffer->bf_getwritebuffer != NULL);
-    }
-#endif
     else
     {
-        PyErr_Clear();
         ptr = PyLong_AsVoidPtr(arg);
 
         if (PyErr_Occurred())
         {
 #if PY_VERSION_HEX >= 0x03010000
-            PyErr_SetString(PyExc_TypeError, "a single integer, CObject, None, buffer protocol implementor or another sip.voidptr object is required");
+            PyErr_SetString(PyExc_TypeError, "a single integer, CObject, None or another sip.voidptr object is required");
 #else
-            PyErr_SetString(PyExc_TypeError, "a single integer, Capsule, CObject, None, buffer protocol implementor or another sip.voidptr object is required");
+            PyErr_SetString(PyExc_TypeError, "a single integer, Capsule, CObject, None or another sip.voidptr object is required");
 #endif
             return 0;
         }
